/* logs.c -- Log copying and archiving.
   $Id: logs.c,v 0.5 1999/04/22 09:05:03 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1999 Board of Trustees, Leland Stanford Jr. University

   This program is free software.  See README included in the distribution
   for terms and conditions.

   Here are the high level functions to copy or archive logs.  Most of the
   real work is done by the functions in file.c, which we call in the
   functions here.  This is the abstraction layer that sits between main.c
   and file.c. */

#include "expand.h"             /* expand() */
#include "file.h"               /* compress(), create_file(), etc. */
#include "logs.h"               /* Prototypes and structs */
#include "utility.h"            /* xmalloc(), syswarn() */

#include <assert.h>             /* assert() */
#include <ctype.h>              /* isdigit() */
#include <dirent.h>             /* DIR, struct dirent, opendir(), etc. */
#include <stdio.h>              /* sscanf() */
#include <stdlib.h>             /* free() */
#include <string.h>             /* strcpy(), strncpy(), strcmp(), etc. */
#include <sys/types.h>          /* time_t */
#include <time.h>               /* difftime() */
#include <unistd.h>             /* access(), F_OK */

/* Linked list of log files to archive, sorted numerically. */
struct logfile {
    char *file;                 /* Name of log file. */
    time_t date;                /* Timestamp of file (from file name). */

    struct logfile *next;
};

/* Stuff for internal use only. */
static struct logfile *find_logs (char *dir, char *base);


/***************************************************************************
* Directory scanning
***************************************************************************/

/* Scan through the given directory, finding all files starting with the
   given base filename followed by at least nine digits.  Insert these into
   a linked list, sorted by the number in the file extension, and return
   that list. */
static struct logfile *find_logs (char *dir, char *base)
{
    DIR *d;
    char *date, *p;
    struct dirent *file;
    int okay, offset;
    struct logfile *log, *current;
    struct logfile *logs = 0;

    d = opendir (dir);
    if (!d) {
        syswarn ("Unable to open directory %s", dir);
        return 0;
    }
    offset = strlen (base);
    while ((file = readdir (d))) {
        if (strlen (file->d_name) < offset + 10) continue;
        if (strncmp (file->d_name, base, offset)) continue;
        if (file->d_name[offset] != '.') continue;
        okay = 1;
        for (p = file->d_name + offset + 1; *p; p++)
            if (!isdigit ((int) *p)) okay = 0;
        if (!okay || p - file->d_name < offset + 10) continue;

        /* We found an old log.  Add it to the list. */
        log = xmalloc (sizeof (struct logfile));
        log->file = xmalloc (strlen (dir) + strlen (file->d_name) + 2);
        log->date = 0;
        strcpy (log->file, dir);
        strcat (log->file, "/");
        strcat (log->file, file->d_name);
        date = strrchr (log->file, '.');
        assert (date != NULL);
        if (sscanf (date + 1, "%lu", &log->date) != 1)
            warn ("Parsing of time from %s failed", date);
        if (!logs || difftime (logs->date, log->date) > 0) {
            log->next = logs;
            logs = log;
        } else {
            for (current = logs; current->next; current = current->next)
                if (difftime (current->next->date, log->date) > 0) break;
            log->next = current->next;
            current->next = log;
        }
    }
    closedir (d);
    return logs;
}


/***************************************************************************
* Interface
***************************************************************************/

/* Take in a log struct for a log that we're just copying.  Since we never
   rename those logs, we can't tell if there was a previous failed attempt,
   so we don't bother looking for old failures. */
void copy_log (struct log *log)
{
    char *path;
    struct archive *current;

    for (current = log->archive; current; current = current->next) {
        path = expand (current->template, log->name, time (NULL));
        if (access (path, F_OK) == 0 && unlink (path) != 0)
            syswarn ("Unable to remove %s", path);
        create_directory (path);
        compress_file (log->path, path);
        free (path);
    }
}


/* We take in a log struct.  From this, we determine the base name of other
   temporary log files and scan the directory the temporary log file is
   located in for other temporary files (old failed log rotations).  We then
   step through the archive instructions for this log and archive all of the
   logs found (in order from oldest to current). */
int archive_logs (struct log *log)
{
    char *directory, *base, *path, *p;
    int okay, old;
    struct archive *current;
    struct logfile *files, *file;

    /* Find the directory all of our logs will be in. */
    p = strrchr (log->path, '/');
    if (!p) {
        directory = ".";
    } else if (p == log->path) {
        directory = "/";
    } else {
        directory = log->path;
        *p = '\0';
    }
    base = p + 1;

    /* Scan the directory for failed log rotations, and then archive all of
       the logs, today's and old failures. */
    files = find_logs (directory, base);
    if (!files) {
        warn ("Unable to find %s", log->tmppath);
        return 0;
    }
    if (!*p) *p = '/';
    for (file = files; file; file = file->next) {
        okay = 1;
        old = 0;
        for (current = log->archive; current; current = current->next) {
            path = expand (current->template, log->name, file->date);
            if (current->count > 0)
                okay &= shuffle_files (path, current->count);
            create_directory (path);
            if (current->count < 0 && access (path, F_OK) == 0)
                if (unlink (path) != 0) {
                    syswarn ("Unable to remove %s", path);
                    okay = 0;
                }
            okay &= compress_file (file->file, path);
            if (!okay) break;
            if (!old && strcmp (log->tmppath, file->file)) {
                old = 1;
                warn ("Archived old log %s", file->file);
            }
            if (old) warn ("   as %s", path);
            free (path);
        }
        if (okay && unlink (file->file) != 0)
            syswarn ("Unable to delete temporary file %s");
    }

    /* Blow away our allocated structures and return success. */
    while (files) {
        file = files;
        files = files->next;
        free (file->file);
        free (file);
    }
    free (directory);
    return 1;
}
