/* main.c -- Log file rotation and periodic maintenance.
   $Id: main.c,v 0.9 1999/04/22 09:05:04 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1997-1999 Board of Trustees, Leland Stanford Jr. University

   This program is free software.  See README included in the distribution
   for terms and conditions.

   This program is designed to rotate, truncate, or otherwise deal with
   system logs on a regular basis, including correctly stopping and
   restarting daemons so that the logs can be moved and compressed safely.
   It takes a configuration file specifying which logs to rotate and what to
   do with them. */

#include "config.h"             /* CONFIG_FILE */
#include "file.h"               /* archive_logs() */
#include "logs.h"               /* read_config(), struct class, etc. */
#include "process.h"            /* run(), hup() */
#include "utility.h"            /* warn() */

#include <stdio.h>              /* printf(), fprintf(), stderr */
#include <stdlib.h>             /* exit() */

/* Package version number, from version.c. */
extern char version[];

/* Stuff for internal use only. */
static void process (struct process *process);
static void usage (int status);


/***************************************************************************
* Documentation
***************************************************************************/

/* Prints out a usage message and exits. */
static void usage (int status)
{
    char usage[] = "Usage: newsyslog [<config-file>]\n";

    if (status > 0) {
        fprintf (stderr, usage);
    } else {
        printf (usage);
        printf (
"\n"
"newsyslog rotates, archives, copies, and/or truncates log files as\n"
"instructed in a configuration file, including running commands before\n"
"and after if desired.  If not given on the command line, the default\n"
"configuration file is %s.\n", CONFIG_FILE);
    }
    exit (status);
}


/***************************************************************************
* Running commands
***************************************************************************/

/* From a process struct, either send HUP to a process or run a command. */
static void process (struct process *process)
{
    if (process->pidfile)
        hup (process->pidfile);
    else if (process->command)
        run (process->command->argv);
}


/***************************************************************************
* Main routine
***************************************************************************/

/* First, read in the configuration file, either the hard-coded value or the
   first command-line argument.  This returns a linked list of log classes.
   Next, step through each log class, calling the stop command if necessary,
   rotate each of the logs in that class, call the restart command, and for
   each log copy it to its archive locations while compressing.

   See logs.h for our data structure here.  We don't bother freeing memory
   as we go; there's no point, since we build up this structure, execute it,
   and then exit. */
int main (int argc, char **argv)
{
    struct class *class, *current;
    struct log *log;
    int i;

    /* Look for --help, -h, --version, and -v command line options. */
    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') break;
        if (argv[i][1] == 'h' || !strcmp (argv[i], "--help")) {
            usage (0);
        } else if (argv[i][1] == 'v' || !strcmp (argv[i], "--version")) {
            printf ("%s\n", version);
            exit (0);
        } else {
            usage (1);
        }
    }
    if (argc > 1 && argc - i > 1) usage (1);

    /* If we have a command-line argument, assume that's the config file. */
    class = read_config (argc > 1 ? argv[i] : CONFIG_FILE);

    for (current = class; current; current = current->next) {
        if (current->stop) process (current->stop);

        /* Rotate all the logs that have archive directives. */
        for (log = current->logs; log; log = log->next) {
            if (!log->rotate) continue;
            log->tmppath = rename_unique (log->path);
            if (log->tmppath)
                create_file (log->path, log->owner, log->group, log->mode);
        }

        if (current->restart) process (current->restart);

        for (log = current->logs; log; log = log->next) {
            if ((!log->tmppath && log->rotate) || !log->archive) {
                warn ("Skipping archiving of %s", log->name);
            } else {
                if (log->rotate) archive_logs (log); else copy_log (log);
            }
        }
    }

    /* Success.  Exit. */
    exit (0);
}
