/* process.c -- Functions for running and signalling processes.
   $Id: process.c,v 0.5 1999/04/22 09:05:04 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1997, 1999 Board of Trustees, Leland Stanford Jr. University

   This program is free software.  See README included in the distribution
   for terms and conditions.

   Some general utility functions for running and signalling processes.  We
   don't want to use system() to run processes due to problems with the
   shell, metacharacters, memory usage, and the like, so we have our own
   function which just does a fork and exec.  There's also a function to
   signal a program that writes a PID file with a SIGHUP to rebind to new
   logs (syslog wants this, for example). */

#include "config.h"             /* pid_t portability */
#include "process.h"            /* Prototypes */
#include "utility.h"            /* warn(), syswarn() */

#include <signal.h>             /* kill(), SIGHUP */
#include <stdio.h>              /* fopen(), fscanf(), fclose(), FILE */
#include <stdlib.h>             /* free() */
#include <sys/types.h>          /* pid_t */
#include <sys/wait.h>           /* wait(), WIFEXITED() */
#include <unistd.h>             /* fork(), execv(), sleep() */


/***************************************************************************
* Program execution
***************************************************************************/

/* Execute a program with fork() and exec() and pass it an arbitrary number
   of arguments.  This is a very limited reimplimentation of something like
   execv() combined with system() which is designed to suit just our
   purposes.  Note that the argv array *has* to be NULL-terminated by the
   caller or evil things happen. */
int run (char **argv)
{
    pid_t child;
    int status;
    int success = 1;

    child = fork ();
    if (child == (pid_t) -1) {
        syswarn ("fork failed");
        success = 0;
    } else if (child == 0) {
        /* In child process, so exec the program. */
        execv (argv[0], argv);
        sysdie ("exec failed");
    } else {
        /* In parent, wait for child to terminate. */
        child = wait (&status);
        if (child < 0) {
            syswarn ("wait failed");
            success = 0;
        }
        if (!WIFEXITED (status)) {
            warn ("%s terminated abnormally", argv[0]);
            success = 0;
        }
    }
    return success;
}


/***************************************************************************
* Signals
***************************************************************************/

/* Send a SIGHUP to a process, reading the PID of the process to SIGHUP from
   a file.  This should be general enough to deal with syslogd and most
   everything else that works similar to it. */
int hup (char *pid_filename)
{
    FILE *pid_file;
    int pid = 0;

    /* Read the PID from the given filename. */
    pid_file = fopen (pid_filename, "r");
    if (pid_file == NULL) {
        warn ("Cannot open PID file %s", pid_filename);
        return 0;
    }
    fscanf (pid_file, "%d", &pid);
    fclose (pid_file);

    /* Careful not to kill process groups and *never* SIGHUP init. */
    if (pid < 2) {
        warn ("Invalid PID %d in file %s", pid, pid_filename);
        return 0;
    }
    if (kill ((pid_t) pid, SIGHUP) != 0) {
        syswarn ("HUP %d from PID file %s failed", pid, pid_filename);
        return 0;
    }

    /* Sleep a couple of seconds to give the daemon time to reset. */
    sleep (2);

    /* Make sure the process is still running. */
    if (kill (pid, 0) != 0) {
        syswarn ("Unable to verify process %d is running", pid);
        warn ("Daemon HUPed from file %s may have died", pid_filename);
        return 0;
    }
    return 1;
}
